#!/bin/bash
# This BMR solution is based on the open-source project Relax-and-Recover (rear, http://relax-and-recover.org/), which
# provides the whole framework for bare metal recovery. Agent provides extra scripts to call agent to do actual
# backup/restore work.
# IMPORTANT PRECONDITIONS:
# 1. User needs to download and install Relax-and-Recover (rear).The required rear version is '2.5' or above.
# 2. User must create and run a backup job in portal and backup necessary files under "/". User needs to ensure backup
#    task is finished successfully and provide the job name to the script.
# 3. User may need to have root privilege.
#
# Default agent defined scripts and default rear scripts location:
# /usr/sbin/rear
# /opt/BUAgent/bmr_iso.sh
# /opt/BUAgent/450_prepare_vvagent.sh   -> copy to  /usr/share/rear/rescue/VV/default/
# /opt/BUAgent/400_restore_backup.sh    -> copy to  /usr/share/rear/restore/VV/default/

# print usage
print_usage() #{{{
{
  echo "Usage: $0 [options]"
  echo " [-p 'full-rear-tool-path']"
  echo "    Full path to the Relax-and-Recover (rear) executable file. Alternatively, enter the full path to the 'REAR_TOOL' environment variable."
  echo " [-b 'backup-task-name']"
  echo "    Name of the BMR backup job. Required for a restore."
  echo " [-r 'restore-vpr-file']"
  echo "    Name of the BMR backup vpr file. Required for a restore."
  echo " [-i 'destination ISO file name]"
  echo "    Create an ISO image and save it to the specified location."
  echo " [-v]"
  echo "    Use verbose mode."
  echo " [-h | -?]"
  echo "    Print a summary of the command usage and exit."
  exit 0
} #}}}

# constant variables
BACKUP_VV="BACKUP=VV"
OUTPUT_ISO="OUTPUT=ISO"
AGENT_DEFAULT_DIR="/opt/BUAgent"          # default agent path
REAR="rear"                               # relax-and-Recover (rear)
REQUIRED_REAR_VERSION="2.5"               # minimum required rear version
ETC="etc/"                                # rear etc folfder, which contains real local configure file
REAR_CONF_FILE="local.conf"               # rear config file
SHARE="share/"                            # rear share folder, which contains real scripts
RESCUE="rescue/"                          # agent && rear rescue folder
RESTORE="restore/"                        # agent && rear restore folder
VV_DEFAULT_PATH="VV/default/"             # relative path of agent defined scripts
ISO_REAR_DIR="/usr/"                      # rear path is fixed as '/usr/sbin/rear' in ISO
PREPARE_AGENT_SH="450_prepare_vvagent.sh" # agent defined rescue script
RESTORE_BACKUP_SH="400_restore_backup.sh" # agent defined script
REAR_DIR="REAR_DIR"
TASK="bmr_task"
VPR_FILE="bmr_0000xxx.vpr"

# local variables
verbose=0                                 # flag to show verbose information
rear_dir=""                               # rear directory
rear_tool=""                              # rear tool
backup_task_name=""                       # backup task name
prepare_restore=0                         # flag to prepare restore process
restore_vpr_file=""                       # restore vpr file
create_iso=0
iso_fullname=""				  # ISO file fullname

# parse arguments
while getopts "b:h?i:p:r:v" opt
do
  case "$opt" in
    b ) backup_task_name="$OPTARG"
        [[ $verbose -eq 1 ]] && echo "Backup job name: '$backup_task_name'."
        ;;
    i ) iso_fullname="$OPTARG"
        create_iso=1
        [[ $verbose -eq 1 ]] && echo "Creating an ISO image: ${iso_fullname}."
        ;;
    r ) restore_vpr_file="$OPTARG"
        prepare_restore=1
        [[ $verbose -eq 1 ]] && echo "Restore with vpr file '$restore_vpr_file'."
        ;;
    v ) verbose=1
        ;;
    p ) rear_tool="$OPTARG"
        echo "Relax-and-Recover (rear) tool: ${rear_tool}."
        ;;
    h | ? ) print_usage
        ;;
  esac
done

# check rear version and config file
check_rear_version_config() #{{{
{
  if [[ -e $rear_tool ]] && [[ -x $rear_tool ]]; then
    # check rear version requirments, eg. "Relax-and-Recover x.x / Git"
    version="$($rear_tool -V)"
    version=$(echo "$version" | cut -d ' ' -f2)
    if [[ $version < $REQUIRED_REAR_VERSION ]]; then
      echo "BMR is not supported with Relax-and-Recover tool version '$version'. The minimum required version is '$REQUIRED_REAR_VERSION'."
      exit 1
    fi

    # modify rear local config file "etc/rear/local.conf" 
    local_conf_file=$rear_dir/../../$ETC$REAR/$REAR_CONF_FILE
    [[ $verbose -eq 1 ]] && echo "Using Relax-and-Recover (rear) local config file '$local_conf_file'."
    if [[ ! -e $local_conf_file ]] ; then
      touch "$local_conf_file"
    fi

    backup_item=$(awk '/BACKUP\=VV/{ print $0 }' "$local_conf_file")
    if [[ -z $backup_item ]] || [[ $backup_item != "$BACKUP_VV" ]]; then
        echo $BACKUP_VV  >> "$local_conf_file"
    fi

    output_item=$(awk '/OUTPUT\=ISO/{ print $0 }' "$local_conf_file")
    if [[ -z $output_item ]] || [[ "$output_item" != "$OUTPUT_ISO" ]]; then
        echo $OUTPUT_ISO  >> "$local_conf_file"
    fi

  else
    echo "Cannot find a Relax-and-Recover (rear) command."
    exit 1
  fi
} #}}}

# set rear directory
set_rear_dir() #{{{
{
  if [[ $prepare_restore -eq 1 ]]; then
    rear_dir=$ISO_REAR_DIR
  else
    # check rear tool
    if [[ -z $rear_tool ]]; then
      # if rear tool is not provided by user, then looking for the environment variable 'REAR_TOOL' or the default rear tool
      rear_tool=${REAR_TOOL:-$(which rear)}
    fi
    [[ $verbose -eq 1 ]] && echo "Using Relax-and-Recover (rear) command '$rear_tool'."
  
    if [[ -n $rear_tool ]]; then
      rear_dir=$(readlink -f "$rear_tool")
      rear_dir=$(dirname "$rear_dir")
    fi
  fi

  [[ $verbose -eq 1 ]] && echo "Using Relax-and-Recover (rear) directory '$rear_dir'."
} #}}}

# check rescue ISO prepare script
bmr_rescure_iso_prepare() #{{{
{
  check_rear_version_config

  # Deleting temporary files
  [[ $verbose -eq 1 ]] && echo "Deleting temporary files ..."
  [[ $verbose -eq 1 ]] && echo "Deleting $rear_dir/../../var/lib/$REAR/output/*"
  rm -Rf $rear_dir/../../var/lib/$REAR/output/*
  [[ $verbose -eq 1 ]] && echo "Deleting /tmp/rear.*."
  rm -Rf /tmp/rear.*

  local isofile=${iso_fullname}
  if [[ -z $iso_fullname ]]; then
    isofile="${agent_dir}/${REAR}/*.iso"
  fi 
  [[ $verbose -eq 1 ]] && echo "Deleting $isofile."
  rm -Rf ${isofile}

  # check rescue source script
  src_rear_rescue_dir="$agent_dir/"
  if [[ ! -e $src_rear_rescue_dir ]] || [[ ! -d $src_rear_rescue_dir ]]; then
    echo "Rescue script missing from '$src_rear_rescue_dir'."
    exit 1
  fi

  # copy rescue VV scripts
  rear_rescue_vv_default_path="$rear_dir/../$SHARE$REAR/$RESCUE$VV_DEFAULT_PATH"
  if [[ ! -e $rear_rescue_vv_default_path ]]; then
    mkdir -p "$rear_rescue_vv_default_path"
  fi

  prepare_agent_sh=$rear_rescue_vv_default_path$PREPARE_AGENT_SH
  [[ $verbose -eq 1 ]] && echo "Copying rescue script $src_rear_rescue_dir$PREPARE_AGENT_SH to $prepare_agent_sh"
  cp -rf "$src_rear_rescue_dir$PREPARE_AGENT_SH" "$prepare_agent_sh"

  # modify rescue VV scripts
  if [[ -e $prepare_agent_sh ]] && [[ $AGENT_DEFAULT_DIR != "$agent_dir" ]]; then
    sed -i "s|$AGENT_DEFAULT_DIR|$agent_dir|g" "$prepare_agent_sh"
  fi
} #}}}

# check restore prepare script
bmr_restore_prepare() #{{{
{
  # need a backup task name
  if [[ -z $backup_task_name ]]; then
    echo "Missing backup job name for restoring files."
    exit 1
  fi

  src_rear_restore_dir="$agent_dir/"
  if [[ ! -e $src_rear_restore_dir ]] || [[ ! -d $src_rear_restore_dir ]]; then
    echo "Missing restore script in '$src_rear_restore_dir'."
    exit 1
  fi

  # copy restore VV scripts
  rear_restore_vv_default_path="$rear_dir/$SHARE$REAR/$RESTORE$VV_DEFAULT_PATH"
  if [[ ! -e $rear_restore_vv_default_path ]]; then
    mkdir -p "$rear_restore_vv_default_path"
  fi

  restore_backup_sh=$rear_restore_vv_default_path$RESTORE_BACKUP_SH
  [[ $verbose -eq 1 ]] && echo "Copying restore script $src_rear_restore_dir$RESTORE_BACKUP_SH to $restore_backup_sh"
  cp -rf "$src_rear_restore_dir$RESTORE_BACKUP_SH" "$restore_backup_sh"

  # check restore VV path
  if [[ -e $restore_backup_sh ]] && [[ $AGENT_DEFAULT_DIR != "$agent_dir" ]]; then
    sed -i "s|$AGENT_DEFAULT_DIR|$agent_dir|g" "$restore_backup_sh"
  fi

  # modify restore VV scripts
  if [[ -e $restore_backup_sh ]]; then
    sed -i "s|$TASK|$backup_task_name|g" "$restore_backup_sh"
    sed -i "s|$VPR_FILE|$restore_vpr_file|g" "$restore_backup_sh"
  fi
} #}}}

# create a full rescue ISO with/without VV restore tasks
run() #{{{
{
  if [[ $create_iso -eq 0 ]] && [[ $prepare_restore -eq 0 ]]; then
    echo "No job is requested."
    print_usage
  fi

  if [[ $create_iso -eq 1 ]] && [[ $prepare_restore -eq 1 ]]; then
    echo "Error: Cannot create ISO and set up restore vpr file simultaneously."
    print_usage
    exit 1;
  fi

  # get agent directory
  agent_dir=$(readlink -f "$(dirname "${0}")")
  [[ $verbose -eq 1 ]] && echo "Using agent installation path '$agent_dir'"

  set_rear_dir

  if [[ $create_iso -eq 1 ]]; then
    bmr_rescure_iso_prepare
    if [[ $verbose -eq 1 ]]; then
      $rear_tool -v mkrescue
    else
      $rear_tool -v mkrescue >/dev/null 2>&1
    fi
    local src=( $(compgen -G "$rear_dir/../../var/lib/$REAR/output/*.iso") )
    # checking the iso file is generated and available
    if [[ -z $src ]]; then
      echo "Error: ISO ${src} file is not found."
      exit 10;
    fi 
    if [[ -z $iso_fullname ]]; then
      iso_fullname="${agent_dir}/${REAR}/."
    fi 
    [[ $verbose -eq 1 ]] && echo "Moving ${src} file to ${iso_fullname}."
    mv $src $iso_fullname
  fi

  # bmr restore prepare can run seperately
  if [[ $prepare_restore -eq 1 ]]; then
    bmr_restore_prepare
  fi
} #}}}

run
